/***
 * Julia
 * Copyright (C) 2005-2006 INRIA, France Telecom, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.juliak.membrane;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.objectweb.fractal.juliak.Juliak;
import org.objectweb.fractal.juliak.platform.Platform;


/**
 * This class defines the membranes supported by Juliak.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @since 2.5
 */
public class JuliakMembranes implements MembraneDefProvider {

    /**
     * Return the definitions of membranes supported by Juliak.
     */
    public MembraneDef[] getMembraneDefs() {
        
        MembraneDef[] basic = new BasicMembranes().getMembraneDefs();
        
        /*
         * Retrieve additional membrane definitions.
         */
        List mdps = getAdditionalMembraneDefs();
        if( mdps == null ) {
            return basic;
        }
        
        List all = new ArrayList();
        all.addAll(Arrays.asList(basic));
        for (Iterator iter = mdps.iterator(); iter.hasNext();) {
            MembraneDefProvider mdp = (MembraneDefProvider) iter.next();
            MembraneDef[] mdef = mdp.getMembraneDefs();
            if( mdef != null ) {
                all.addAll(Arrays.asList(mdef));
            }
        }
        
        return (MembraneDef[]) all.toArray(new MembraneDef[all.size()]);
    }
    
    
    // ---------------------------------------------------------
    // Implementation specific
    // ---------------------------------------------------------
    
    /**
     * Return the list of additional membrane definition providers which can be
     * specified with the {@link Juliak#CONFIG_PROP_NAME} system property.
     */
    private static List getAdditionalMembraneDefs() {

        String add = System.getProperty(Juliak.CONFIG_PROP_NAME);
        if( add==null || add.length()==0 ) {
            return null;
        }
        
        List mdps = new ArrayList();
        StringTokenizer st = new StringTokenizer(add);
        while( st.hasMoreTokens() ) {
            String name = st.nextToken();
            Object o = loadAndInstanciate(name);
            if( ! (o instanceof MembraneDefProvider) ) {
                final String msg =
                    "Membrane definition class "+name+" should implement "+
                    MembraneDefProvider.class.getName()+
                    ". Check system property "+Juliak.CONFIG_PROP_NAME;
                throw new RuntimeException(msg);
            }
            MembraneDefProvider mdp = (MembraneDefProvider) o;
            mdps.add(mdp);
        }
        
        return mdps;
    }
    
    private static Object loadAndInstanciate( String name ) {
        try {
            Class cl = Platform.get().loadClass(name);
            Object o = cl.newInstance();
            return o;
        }
        catch( ClassNotFoundException e ) {
            final String msg = e.getClass()+": "+e.getMessage();
            throw new RuntimeException(msg);
        } catch (InstantiationException e) {
            final String msg = e.getClass()+": "+e.getMessage();
            throw new RuntimeException(msg);
        } catch (IllegalAccessException e) {
            final String msg = e.getClass()+": "+e.getMessage();
            throw new RuntimeException(msg);
        }        
    }
}
